@@ -21,6 +21,7 @@ import operator
 import progressbar as pg
 import scipy.cluster.hierarchy as sch
 from numba import jit, prange
+from hashlib import md5
 
 
 try:
@@ -262,7 +263,7 @@ def return_selection_atom(use_for, traj, args):
     return selection
 
 
-def save_dist_mat(distmat, rmsd_string):
+def save_dist_mat(distmat, rmsd_string, maxtrixtype):
     """
     DESCRIPTION
     Save the numpy matrix to reused afterward
@@ -275,10 +276,20 @@ def save_dist_mat(distmat, rmsd_string):
     """
     if rmsd_string:
         name = rmsd_string.replace(" ", "_")
+
     else:
         name = "matrix_all"
-    np.save(name, distmat)
-    printScreenLogfile("Saving distance matrix : {0}.npy".format(name))
+
+    try:
+        np.save(name, distmat)
+        printScreenLogfile("Saving {0} : {1}.npy".format(maxtrixtype, name))
+    except OSError as error:
+        print("Selection line too long to be used as filename. TTClust will hash it")
+        namehashed = md5(name.encode()).hexdigest()
+        print(f"{name} -> {namehashed}")
+        np.save(namehashed, distmat)
+        printScreenLogfile("Saving {0}: {1}.npy".format(maxtrixtype, namehashed))
+
 
 
 def reorder_cluster(clusters):
@@ -412,7 +423,7 @@ def ask_choice(args, name):
         npy_files = glob.glob("*.npy")
         for i, file in enumerate(npy_files):
             print("  {0} - {1}".format(i + 1, file))
-        print(" -->Please chooce and press Enter")
+        print(" -->Please choose and press Enter")
         # Check if the user give a good answer
         choice_file = input()
         try:
@@ -440,12 +451,18 @@ def search_dist_mat(rmsd_string, args):
     else:
         name = "matrix_all"
     # Searching all npy file in the folder
-    npy_files = glob.glob("*.npy")
+    namehashed = md5(name.encode()).hexdigest()
+    namehashed += '.npy'
+
     if not name[-4:] == ".npy":
         name += ".npy"
 
+    npy_files = glob.glob("*.npy")
+
     if name in npy_files and not args["interactive"].lower() == "n":
         return ask_choice(args, name)
+    elif namehashed in npy_files and not args["interactive"].lower() == "n":
+        return ask_choice(args, namehashed)
     else:
         return None
 
@@ -522,8 +539,8 @@ def create_DM(traj, args):
                                                      traj=traj,
                                                      args=args)
 
-        RMSD_selection = return_selection_atom("RMSD", traj=traj,
-                                               args=args)
+        # RMSD_selection = return_selection_atom("RMSD", traj=traj,
+        #                                        args=args)
 
         # Trajectory superposition  (aligment)
         traj_aligned = traj.superpose(traj[0],
@@ -532,15 +549,18 @@ def create_DM(traj, args):
     else:
         traj_aligned = traj
 
-    select_align = improve_nucleic_acid(args["select_alignement"])
+    # select_align = improve_nucleic_acid(args["select_alignement"])
+    # Transform the RMSD string for nucleic acids.
     untouch_rmsd_string = args["select_rmsd"]
     rmsd_string = improve_nucleic_acid(args["select_rmsd"])
 
+    # If the RMSD string is not empty
     if rmsd_string:
         print("NOTE : Extraction of subtrajectory for time optimisation")
+        # Extract the subpart.
         traj_aligned = extract_selected_atoms(rmsd_string, traj_aligned, args["logname"])
     # matrix initialization
-    distances = np.empty((traj.n_frames, traj.n_frames))
+    distances = np.zeros((traj.n_frames, traj.n_frames),dtype=np.float32)
 
     # Searching if a distance file already exist
     distance_file = search_dist_mat(untouch_rmsd_string, args)
@@ -551,14 +571,14 @@ def create_DM(traj, args):
         return np.load(distance_file)
     else:  # otherwise
 
-        pbar = pg.ProgressBar(widgets=WIDGETS, maxval=traj.n_frames).start()
+        pbar = pg.ProgressBar(widgets=WIDGETS, maxval=traj_aligned.n_frames).start()
         counter = 0
         # Pairwise RMSD calculation (matrix n²)
-        for i in range(traj.n_frames):
+        for i in range(traj_aligned.n_frames):
             # distances[i] = md.rmsd(traj_aligned, traj_aligned, frame=i)
 
-            for j in range(i + 1, traj.n_frames):
-                rmsd = calc_rmsd_2frames(traj.xyz[i], traj.xyz[j])
+            for j in range(i + 1, traj_aligned.n_frames):
+                rmsd = calc_rmsd_2frames(traj_aligned.xyz[i], traj_aligned.xyz[j])
                 distances[i][j] = rmsd
                 distances[j][i] = rmsd
             pbar.update(counter)
@@ -568,7 +588,7 @@ def create_DM(traj, args):
 
         # Finaly, we save the matrix if we want to load it again afterward
         print("Calculation ended - saving distance matrix")
-        save_dist_mat(distances, args["select_rmsd"])
+        save_dist_mat(distances, args["select_rmsd"], "distance matrix")
 
         return distances
 
@@ -693,7 +713,7 @@ def create_cluster_table(traj, args):
     if select_rmsd == None:
         select_rmsd = "None"
 
-    linkage_file = search_dist_mat(untouch_select_rmsd + " linkage " + args["method"], args)
+    linkage_file = search_dist_mat(select_align+'--'+untouch_select_rmsd + " linkage " + args["method"], args)
 
     if linkage_file:
         linkage = np.load(linkage_file)
@@ -711,7 +731,7 @@ def create_cluster_table(traj, args):
             sys.exit(1)
         print("         >Done!")
         print("         ...Saving linkage matrix...")
-        save_dist_mat(linkage, untouch_select_rmsd + " linkage " + args["method"])
+        save_dist_mat(linkage, select_align+'--'+untouch_select_rmsd + " linkage " + args["method"], "linkage matrix")
         print("         >Done!")
 
     # if a cuttof for distance cuting is given
@@ -942,7 +962,7 @@ def plot_dendro(linkage, logname, cutoff, color_list, clusters_list):
         STYLE = "classic"
         if STYLE in plt.style.available:
             plt.style.use(STYLE)
-    fig = plt.figure()
+    fig,ax = plt.subplots()
     # Convert RGB color to HEX color
     color_hex = [mpl.colors.rgb2hex(x) for x in color_list]
     sch.set_link_color_palette(color_hex)
@@ -966,7 +986,6 @@ def plot_dendro(linkage, logname, cutoff, color_list, clusters_list):
 
     # Graph parameters
     plt.title("Clustering Dendrogram")
-    ax = plt.axes()
     ax.set_xticklabels([])
     plt.axhline(y=float(cutoff), color="grey")  # cutoff value vertical line
     ax.set_ylabel("Distance (AU)")
@@ -1272,7 +1291,6 @@ def define_LOGFILE(log):
     global LOGFILE
     LOGFILE = log
 
-
 def main():
     """
     Execute TTclust
